// Contains 5 detector grids and newNote detectors
// Coded by Jeff Yates and Tom Gowing
// Cornell University Spring 2011
// *******************************************************************************
module Bar_Detector (
	iCLK_27,
	iNewFrame,
	iFrameCount,
	iRed, iGreen, iBlue,
	iVGA_X, iVGA_Y,
	iVGA_BLANK,
	iGxy, iRxy, iYxy, iBxy, iOxy,
	oNotes,
	oFrameNum,
	oWriteBuffer );
	
	input				iCLK_27;
	input				iNewFrame;				// Signals start of new frame
	input	[7:0]		iFrameCount;			// Global frame count
	input	[9:0]		iRed, iGreen, iBlue;	// RGB signal from decoder
	input	[9:0]		iVGA_X, iVGA_Y;		// Current pixel location
	input				iVGA_BLANK;				// High when VGA is reading pixels
	input	[19:0] 	iGxy,iRxy,iYxy,iBxy,iOxy;	// Bottom-left coordinate for each detector grid
	
	output reg	[4:0]	oNotes;			// Notes to strum
	output 		[7:0]	oFrameNum;		// Frame to strum on
	output reg 			oWriteBuffer;	// Tells buffer to write
	
	parameter GREEN = 3'd0, RED = 3'd1, YELLOW = 3'd2, BLUE = 3'd3, ORANGE = 3'd4;
	parameter DELAY = 8'd20;	// Hard coded frame delay between detection and strum (8 if wired) (4 if 1P wireless)

	
	wire 	newGr, newRe, newYe, newBl, newOr;	// New detection of the note
	wire	Gr, Re, Ye, Bl, Or;
	
	assign oFrameNum = iFrameCount + DELAY;
		
	// Signal FIFO to write when newNote detected and update notes output
	always @(posedge iCLK_27) begin
		if (iNewFrame) begin
			oWriteBuffer <= ((newGr|newRe|newYe|newBl|newOr)) ? 1'b1 : 1'b0;
		end
		else begin
			oWriteBuffer <= 1'b0;
		end
		
		oNotes <= {newGr,newRe,newYe,newBl,newOr};
	end
		
	// 1 Grid detector for each note**********************************************
	Grid_Detector Green_D (
		.iCLK_27		(iCLK_27),
		.iRed			(iRed),
		.iGreen		(iGreen),
		.iBlue		(iBlue),
		.iVGA_X		(iVGA_X),
		.iVGA_Y		(iVGA_Y),
		.iVGA_BLANK	(iVGA_BLANK),
		.iBotLeft	(iGxy),
		.iColorSel	(GREEN),
		.oColor		(Gr)
	);
	Grid_Detector Red_D (
		.iCLK_27		(iCLK_27),
		.iRed			(iRed),
		.iGreen		(iGreen),
		.iBlue		(iBlue),
		.iVGA_X		(iVGA_X),
		.iVGA_Y		(iVGA_Y),
		.iVGA_BLANK	(iVGA_BLANK),
		.iBotLeft	(iRxy),
		.iColorSel	(RED),
		.oColor		(Re)
	);
	Grid_Detector Yellow_D (
		.iCLK_27		(iCLK_27),
		.iRed			(iRed),
		.iGreen		(iGreen),
		.iBlue		(iBlue),
		.iVGA_X		(iVGA_X),
		.iVGA_Y		(iVGA_Y),
		.iVGA_BLANK	(iVGA_BLANK),
		.iBotLeft	(iYxy),
		.iColorSel	(YELLOW),
		.oColor		(Ye)
	);
	Grid_Detector Blue_D (
		.iCLK_27		(iCLK_27),
		.iRed			(iRed),
		.iGreen		(iGreen),
		.iBlue		(iBlue),
		.iVGA_X		(iVGA_X),
		.iVGA_Y		(iVGA_Y),
		.iVGA_BLANK	(iVGA_BLANK),
		.iBotLeft	(iBxy),
		.iColorSel	(BLUE),
		.oColor		(Bl)
	);
	Grid_Detector Orange_D (
		.iCLK_27		(iCLK_27),
		.iRed			(iRed),
		.iGreen		(iGreen),
		.iBlue		(iBlue),
		.iVGA_X		(iVGA_X),
		.iVGA_Y		(iVGA_Y),
		.iVGA_BLANK	(iVGA_BLANK),
		.iBotLeft	(iOxy),
		.iColorSel	(ORANGE),
		.oColor		(Or)
	);
	
	// Detect newNote for each note**********************************************
	newNote np_green (
		.iCLK			(iCLK_27),
		.iNote		(Gr),
		.iNewFrame	(iNewFrame),
		.oNewNote	(newGr)
	);
	newNote np_red (
		.iCLK			(iCLK_27),
		.iNote		(Re),
		.iNewFrame	(iNewFrame),
		.oNewNote	(newRe)
	);
	newNote np_yellow (
		.iCLK			(iCLK_27),
		.iNote		(Ye),
		.iNewFrame	(iNewFrame),
		.oNewNote	(newYe)
	);
	newNote np_blue (
		.iCLK			(iCLK_27),
		.iNote		(Bl),
		.iNewFrame	(iNewFrame),
		.oNewNote	(newBl)
	);
	newNote np_orange (
		.iCLK			(iCLK_27),
		.iNote		(Or),
		.iNewFrame	(iNewFrame),
		.oNewNote	(newOr)
	);
	
endmodule
