// Contains 4 detector grids and newNote detectors
// Coded by Jeff Yates and Tom Gowing
// Cornell University Spring 2011
// *******************************************************************************
module Bar_Detector_Drum (
	iCLK_27,
	iNewFrame,
	iFrameCount,
	iRed, iGreen, iBlue,
	iVGA_X, iVGA_Y,
	iVGA_BLANK,
	iGxy, iRxy, iYxy, iBxy,
	oNotes,
	oPedals,
	oPedalLED,
	oFrameNum,
	oWriteBuffer );
	
	input				iCLK_27;
	input				iNewFrame;				// Signals start of new frame
	input	[7:0]		iFrameCount;			// Global frame count
	input	[9:0]		iRed, iGreen, iBlue;	// RGB signal from decoder
	input	[9:0]		iVGA_X, iVGA_Y;		// Current pixel location
	input				iVGA_BLANK;				// High when VGA is reading pixels
	input	[19:0] 	iGxy,iRxy,iYxy,iBxy;	// Bottom-left coordinate for each detector grid
	
	output reg	[3:0]	oNotes;			// Notes to hit
	output reg	[3:0]	oPedals;			// Outputs of the Pedal signals from the 4 detectors
	output reg	[3:0]	oPedalLED;		// Outputs of the Pedal signals from the 4 detectors for LED debugging
	output 		[7:0]	oFrameNum;		// Frame to hit notes on
	output reg 			oWriteBuffer;	// Tells buffer to write
	
	parameter GREEN = 3'd0, RED = 3'd1, YELLOW = 3'd2, BLUE = 3'd3;
	parameter DELAY = 8'd20;	// Hard coded frame delay between detection and strum
	parameter GUITAR = 2'd0, BASS = 2'd1, DRUM = 2'd2;

	
	wire 	newGr, newRe, newYe, newBl;			// New detection of the note
	wire 	newGr_P, newRe_P, newYe_P, newBl_P;	// New detection of the pedal
	wire	Gr, Re, Ye, Bl;
	wire	Gr_P, Re_P, Ye_P, Bl_P;
	
	assign oFrameNum = iFrameCount + DELAY;
	
	// Signal FIFO to write when newNote detected and update notes output
	always @(posedge iCLK_27) begin
		if (iNewFrame) begin
			oWriteBuffer <= (((newGr_P|newRe_P|newYe_P|newBl_P)|(newGr|newRe|newYe|newBl))) ? 1'b1 : 1'b0;
		end
		else begin
			oWriteBuffer <= 1'b0;
		end
		
		oNotes <= {newGr,newRe,newYe,newBl};
		oPedals <= {newGr_P,newRe_P,newYe_P,newBl_P};
		oPedalLED <= {Re_P,Ye_P,Bl_P,Gr_P};
	end
		
	// 1 Grid detector for each note**********************************************
	Grid_Detector_Drum Green_D (
		.iCLK_27		(iCLK_27),
		.iRed			(iRed),
		.iGreen		(iGreen),
		.iBlue		(iBlue),
		.iVGA_X		(iVGA_X),
		.iVGA_Y		(iVGA_Y),
		.iVGA_BLANK	(iVGA_BLANK),
		.iBotLeft	(iGxy),
		.iColorSel	(GREEN),
		.oColor		(Gr),
		.oPedal		(Gr_P)
	);
	Grid_Detector_Drum Red_D (
		.iCLK_27		(iCLK_27),
		.iRed			(iRed),
		.iGreen		(iGreen),
		.iBlue		(iBlue),
		.iVGA_X		(iVGA_X),
		.iVGA_Y		(iVGA_Y),
		.iVGA_BLANK	(iVGA_BLANK),
		.iBotLeft	(iRxy),
		.iColorSel	(RED),
		.oColor		(Re),
		.oPedal		(Re_P)
	);
	Grid_Detector_Drum Yellow_D (
		.iCLK_27		(iCLK_27),
		.iRed			(iRed),
		.iGreen		(iGreen),
		.iBlue		(iBlue),
		.iVGA_X		(iVGA_X),
		.iVGA_Y		(iVGA_Y),
		.iVGA_BLANK	(iVGA_BLANK),
		.iBotLeft	(iYxy),
		.iColorSel	(YELLOW),
		.oColor		(Ye),
		.oPedal		(Ye_P)
	);
	Grid_Detector_Drum Blue_D (
		.iCLK_27		(iCLK_27),
		.iRed			(iRed),
		.iGreen		(iGreen),
		.iBlue		(iBlue),
		.iVGA_X		(iVGA_X),
		.iVGA_Y		(iVGA_Y),
		.iVGA_BLANK	(iVGA_BLANK),
		.iBotLeft	(iBxy),
		.iColorSel	(BLUE),
		.oColor		(Bl),
		.oPedal		(Bl_P)
	);
	
	// Detect newNote for each note**********************************************
	newNote np_green (
		.iCLK			(iCLK_27),
		.iNote		(Gr),
		.iNewFrame	(iNewFrame),
		.oNewNote	(newGr)
	);
	newNote np_red (
		.iCLK			(iCLK_27),
		.iNote		(Re),
		.iNewFrame	(iNewFrame),
		.oNewNote	(newRe)
	);
	newNote np_yellow (
		.iCLK			(iCLK_27),
		.iNote		(Ye),
		.iNewFrame	(iNewFrame),
		.oNewNote	(newYe)
	);
	newNote np_blue (
		.iCLK			(iCLK_27),
		.iNote		(Bl),
		.iNewFrame	(iNewFrame),
		.oNewNote	(newBl)
	);
	// Detect newPress for Pedal values
	newNote np_greenP (
		.iCLK			(iCLK_27),
		.iNote		(Gr_P),
		.iNewFrame	(iNewFrame),
		.oNewNote	(newGr_P)
	);
	newNote np_redP (
		.iCLK			(iCLK_27),
		.iNote		(Re_P),
		.iNewFrame	(iNewFrame),
		.oNewNote	(newRe_P)
	);
	newNote np_yellowP (
		.iCLK			(iCLK_27),
		.iNote		(Ye_P),
		.iNewFrame	(iNewFrame),
		.oNewNote	(newYe_P)
	);
	newNote np_blueP (
		.iCLK			(iCLK_27),
		.iNote		(Bl_P),
		.iNewFrame	(iNewFrame),
		.oNewNote	(newBl_P)
	);
	
endmodule
