//////////////////////////////////////////////////////////
//Cursor module displays a cursor on the screen.
//R, G or B values are outputted to LEDR based on SW[4:2].
//Cursor is displayed on 7SEGs in decimal.
//////////////////////////////////////////////////////////
module Cursor (
	iCLK,
	iReset,
	iVGA_X, iVGA_Y,
	iRed,	iGreen, iBlue,
	iRGBsel,
	iMoveCursor,
	iCursorDir,
	iGxy_G,iRxy_G,iYxy_G,iBxy_G,iOxy_G,
	iGxy_B,iRxy_B,iYxy_B,iBxy_B,iOxy_B,
	iGxy_D,iRxy_D,iYxy_D,iBxy_D,
	iPresetSwitches,
	oRed,	oGreen, oBlue,
	oColor,
	oCursorX, oCursorY );
	
	input				iCLK;						//27MHz TV Clock
	input				iReset;					//Module reset from KEY 0
	input	[9:0] 	iVGA_X, iVGA_Y;		//Coordinates requested by VGA_Ctrl
	input	[9:0]		iRed, iGreen, iBlue;	//RGB from SRAM
	input	[2:0]		iRGBsel;					//Selects to display R, G, or B on LEDs
	input				iMoveCursor;			//Update the cursor from KEY 3 press
	input	[4:0]		iCursorDir;				//Which direction to move cursor from SW[4:0] UDLR
	input [19:0]	iGxy_G,iRxy_G,iYxy_G,iBxy_G,iOxy_G;	//Bottom-Left coordinate for detection boxes
	input [19:0]	iGxy_B,iRxy_B,iYxy_B,iBxy_B,iOxy_B;	//Bottom-Left coordinate for detection boxes
	input [19:0]	iGxy_D,iRxy_D,iYxy_D,iBxy_D;			//Bottom-Left coordinate for detection boxes
	input [4:0] 	iPresetSwitches;		//Selects 1 of 5 preset cursor coordinates
	
	output		[9:0]	oRed, oGreen, oBlue;	//RGB output to VGA_Ctrl
	output reg	[9:0]	oColor;					//Color output to LEDs
	output		[9:0] oCursorX, oCursorY;	//Cursor location
	
	reg [9:0]	CursorX, CursorY;
	
	reg [9:0]	Cursor_R; //store the Red portion of the color of every pixel in the cursor
	reg [9:0]	Cursor_G; //store the Red portion of the color of every pixel in the cursor
	reg [9:0]	Cursor_B; //store the Red portion of the color of every pixel in the cursor
	
	wire			updateCursor;	//Debounced signal to update cursor location
	
	assign oCursorX = CursorX;
	assign oCursorY = CursorY;
	
	parameter width = 10'd3, height = 10'd3;
	
	wire center;
	
	assign center = iVGA_X==CursorX && iVGA_Y==CursorY;
	
	//Display detection boxes for Guitar (left-player)
	wire [9:0] Gx_G,Gy_G,Rx_G,Ry_G,Yx_G,Yy_G,Bx_G,By_G,Ox_G,Oy_G;
	assign Gx_G = iGxy_G[19:10]; assign Gy_G = iGxy_G[9:0];
	assign Rx_G = iRxy_G[19:10]; assign Ry_G = iRxy_G[9:0];
	assign Yx_G = iYxy_G[19:10]; assign Yy_G = iYxy_G[9:0];
	assign Bx_G = iBxy_G[19:10]; assign By_G = iBxy_G[9:0];
	assign Ox_G = iOxy_G[19:10]; assign Oy_G = iOxy_G[9:0];
	
	wire Gdetect_G, Rdetect_G, Ydetect_G, Bdetect_G, Odetect_G;
	assign Gdetect_G = ((iVGA_Y==(Gy_G-4'd1) && iVGA_X==Gx_G) || (iVGA_Y==(Gy_G-4'd1) && iVGA_X==(Gx_G+10'd2)) ||
							(iVGA_Y==Gy_G && iVGA_X<=(Gx_G+10'd2) && iVGA_X>=Gx_G) ||
							(iVGA_Y==(Gy_G-4'd2) && iVGA_X<=(Gx_G+10'd2) && iVGA_X>=Gx_G));
	assign Rdetect_G = ((iVGA_Y==(Ry_G-4'd1) && iVGA_X==Rx_G) || (iVGA_Y==(Ry_G-4'd1) && iVGA_X==(Rx_G+10'd2)) ||
							(iVGA_Y==Ry_G && iVGA_X<=(Rx_G+10'd2) && iVGA_X>=Rx_G) ||
							(iVGA_Y==(Ry_G-4'd2) && iVGA_X<=(Rx_G+10'd2) && iVGA_X>=Rx_G));
	assign Ydetect_G = ((iVGA_Y==(Yy_G-4'd1) && iVGA_X==Yx_G) || (iVGA_Y==(Yy_G-4'd1) && iVGA_X==(Yx_G+10'd2)) ||
							(iVGA_Y==Yy_G && iVGA_X<=(Yx_G+10'd2) && iVGA_X>=Yx_G) ||
							(iVGA_Y==(Yy_G-4'd2) && iVGA_X<=(Yx_G+10'd2) && iVGA_X>=Yx_G));
	assign Bdetect_G = ((iVGA_Y==(By_G-4'd1) && iVGA_X==Bx_G) || (iVGA_Y==(By_G-4'd1) && iVGA_X==(Bx_G+10'd2)) ||
							(iVGA_Y==By_G && iVGA_X<=(Bx_G+10'd2) && iVGA_X>=Bx_G) ||
							(iVGA_Y==(By_G-4'd2) && iVGA_X<=(Bx_G+10'd2) && iVGA_X>=Bx_G));
	assign Odetect_G = ((iVGA_Y==(Oy_G-4'd1) && iVGA_X==Ox_G) || (iVGA_Y==(Oy_G-4'd1) && iVGA_X==(Ox_G+10'd2)) ||
							(iVGA_Y==Oy_G && iVGA_X<=(Ox_G+10'd2) && iVGA_X>=Ox_G) ||
							(iVGA_Y==(Oy_G-4'd2) && iVGA_X<=(Ox_G+10'd2) && iVGA_X>=Ox_G));
	//Display detection boxes for Bass (right-player)
	wire [9:0] Gx_B,Gy_B,Rx_B,Ry_B,Yx_B,Yy_B,Bx_B,By_B,Ox_B,Oy_B;
	assign Gx_B = iGxy_B[19:10]; assign Gy_B = iGxy_B[9:0];
	assign Rx_B = iRxy_B[19:10]; assign Ry_B = iRxy_B[9:0];
	assign Yx_B = iYxy_B[19:10]; assign Yy_B = iYxy_B[9:0];
	assign Bx_B = iBxy_B[19:10]; assign By_B = iBxy_B[9:0];
	assign Ox_B = iOxy_B[19:10]; assign Oy_B = iOxy_B[9:0];
	
	wire Gdetect_B, Rdetect_B, Ydetect_B, Bdetect_B, Odetect_B;
	assign Gdetect_B = ((iVGA_Y==(Gy_B-4'd1) && iVGA_X==Gx_B) || (iVGA_Y==(Gy_B-4'd1) && iVGA_X==(Gx_B+10'd2)) ||
							(iVGA_Y==Gy_B && iVGA_X<=(Gx_B+10'd2) && iVGA_X>=Gx_B) ||
							(iVGA_Y==(Gy_B-4'd2) && iVGA_X<=(Gx_B+10'd2) && iVGA_X>=Gx_B));
	assign Rdetect_B = ((iVGA_Y==(Ry_B-4'd1) && iVGA_X==Rx_B) || (iVGA_Y==(Ry_B-4'd1) && iVGA_X==(Rx_B+10'd2)) ||
							(iVGA_Y==Ry_B && iVGA_X<=(Rx_B+10'd2) && iVGA_X>=Rx_B) ||
							(iVGA_Y==(Ry_B-4'd2) && iVGA_X<=(Rx_B+10'd2) && iVGA_X>=Rx_B));
	assign Ydetect_B = ((iVGA_Y==(Yy_B-4'd1) && iVGA_X==Yx_B) || (iVGA_Y==(Yy_B-4'd1) && iVGA_X==(Yx_B+10'd2)) ||
							(iVGA_Y==Yy_B && iVGA_X<=(Yx_B+10'd2) && iVGA_X>=Yx_B) ||
							(iVGA_Y==(Yy_B-4'd2) && iVGA_X<=(Yx_B+10'd2) && iVGA_X>=Yx_B));
	assign Bdetect_B = ((iVGA_Y==(By_B-4'd1) && iVGA_X==Bx_B) || (iVGA_Y==(By_B-4'd1) && iVGA_X==(Bx_B+10'd2)) ||
							(iVGA_Y==By_B && iVGA_X<=(Bx_B+10'd2) && iVGA_X>=Bx_B) ||
							(iVGA_Y==(By_B-4'd2) && iVGA_X<=(Bx_B+10'd2) && iVGA_X>=Bx_B));
	assign Odetect_B = ((iVGA_Y==(Oy_B-4'd1) && iVGA_X==Ox_B) || (iVGA_Y==(Oy_B-4'd1) && iVGA_X==(Ox_B+10'd2)) ||
							(iVGA_Y==Oy_B && iVGA_X<=(Ox_B+10'd2) && iVGA_X>=Ox_B) ||
							(iVGA_Y==(Oy_B-4'd2) && iVGA_X<=(Ox_B+10'd2) && iVGA_X>=Ox_B));
	//Display detection boxes for Drums (center-player)
	wire [9:0] Gx_D,Gy_D,Rx_D,Ry_D,Yx_D,Yy_D,Bx_D,By_D;
	assign Gx_D = iGxy_D[19:10]; assign Gy_D = iGxy_D[9:0];
	assign Rx_D = iRxy_D[19:10]; assign Ry_D = iRxy_D[9:0];
	assign Yx_D = iYxy_D[19:10]; assign Yy_D = iYxy_D[9:0];
	assign Bx_D = iBxy_D[19:10]; assign By_D = iBxy_D[9:0];
	
	wire Gdetect_D, Rdetect_D, Ydetect_D, Bdetect_D, Odetect_D;
	assign Gdetect_D = ((iVGA_Y==(Gy_D-4'd1) && iVGA_X==Gx_D) || (iVGA_Y==(Gy_D-4'd1) && iVGA_X==(Gx_D+10'd2)) ||
							(iVGA_Y==Gy_D && iVGA_X<=(Gx_D+10'd2) && iVGA_X>=Gx_D) ||
							(iVGA_Y==(Gy_D-4'd2) && iVGA_X<=(Gx_D+10'd2) && iVGA_X>=Gx_D));
	assign Rdetect_D = ((iVGA_Y==(Ry_D-4'd1) && iVGA_X==Rx_D) || (iVGA_Y==(Ry_D-4'd1) && iVGA_X==(Rx_D+10'd2)) ||
							(iVGA_Y==Ry_D && iVGA_X<=(Rx_D+10'd2) && iVGA_X>=Rx_D) ||
							(iVGA_Y==(Ry_D-4'd2) && iVGA_X<=(Rx_D+10'd2) && iVGA_X>=Rx_D));
	assign Ydetect_D = ((iVGA_Y==(Yy_D-4'd1) && iVGA_X==Yx_D) || (iVGA_Y==(Yy_D-4'd1) && iVGA_X==(Yx_D+10'd2)) ||
							(iVGA_Y==Yy_D && iVGA_X<=(Yx_D+10'd2) && iVGA_X>=Yx_D) ||
							(iVGA_Y==(Yy_D-4'd2) && iVGA_X<=(Yx_D+10'd2) && iVGA_X>=Yx_D));
	assign Bdetect_D = ((iVGA_Y==(By_D-4'd1) && iVGA_X==Bx_D) || (iVGA_Y==(By_D-4'd1) && iVGA_X==(Bx_D+10'd2)) ||
							(iVGA_Y==By_D && iVGA_X<=(Bx_D+10'd2) && iVGA_X>=Bx_D) ||
							(iVGA_Y==(By_D-4'd2) && iVGA_X<=(Bx_D+10'd2) && iVGA_X>=Bx_D));
							
	wire Gdetect, Rdetect, Ydetect, Bdetect, Odetect;
	assign Gdetect = Gdetect_G | Gdetect_B | Gdetect_D;
	assign Rdetect = Rdetect_G | Rdetect_B | Rdetect_D;
	assign Ydetect = Ydetect_G | Ydetect_B | Ydetect_D;
	assign Bdetect = Bdetect_G | Bdetect_B | Bdetect_D;
	assign Odetect = Odetect_G | Odetect_B;
							
	assign oRed 	= ((center) ? {10{1'b0}} : (
						  (Gdetect|Rdetect|Ydetect|Bdetect|Odetect) ? {10{1'b1}} : iRed));
	assign oGreen 	= ((center) ? {10{1'b0}} : (
						  (Gdetect|Rdetect|Ydetect|Bdetect|Odetect) ? {10{1'b1}} : iGreen));
	assign oBlue 	= ((center) ? {10{1'b0}} : (
						  (Gdetect|Rdetect|Ydetect|Bdetect|Odetect) ? {10{1'b1}} : iBlue));
	
	//Debounce move cursor press to determine if its a new press.
	newPress np0(iCLK, iMoveCursor, updateCursor);
	
	always @(posedge iCLK) begin
		if (iReset) begin	
			//Reset cursor to not middle of screen
			CursorX <= Gx_G;
			CursorY <= Gy_G;
		end
		else begin
			if (iVGA_X==(CursorX) && iVGA_Y==(CursorY)) begin
				Cursor_R <= iRed;
				Cursor_G <= iGreen;
				Cursor_B <= iBlue;
			end
			//Assign oColor based on RGB select switches (after entire cursor has been stored)
			//Output the middle pixel value for pinpoint precision
			if (iVGA_X==(10'd0) && iVGA_Y==(10'd0)) begin
				oColor <= iRGBsel[2] ? Cursor_R : 
							 iRGBsel[1] ? Cursor_G : 
							 iRGBsel[0] ? Cursor_B : 10'd0;
			end
			else begin
				oColor <= oColor;	//Hold value until next frame update
			end			
			
			//Select preset cursor locations with SW[9:5] and KEY[3]
			if (updateCursor && (iPresetSwitches != 5'b0)) begin
				CursorX <= iPresetSwitches[4] ? iGxy_G[19:10] :					//SW[9] is Green
							  iPresetSwitches[3] ? iRxy_G[19:10] :					//SW[8] is Red
							  iPresetSwitches[2] ? iYxy_G[19:10] :					//SW[7] is Yellow
							  iPresetSwitches[1] ? iBxy_G[19:10] :					//SW[6] is Blue
							  iPresetSwitches[0] ? iOxy_G[19:10] : CursorX;		//SW[5] is Orange
							  
				CursorY <= iPresetSwitches[4] ? iGxy_G[9:0] :					//SW[9] is Green
							  iPresetSwitches[3] ? iRxy_G[9:0] :					//SW[8] is Red
							  iPresetSwitches[2] ? iYxy_G[9:0] :					//SW[7] is Yellow
							  iPresetSwitches[1] ? iBxy_G[9:0] :					//SW[6] is Blue
							  iPresetSwitches[0] ? iOxy_G[9:0] : CursorY;		//SW[5] is Orange
			end
			//Handle cursor movement from user
			else if (updateCursor && iCursorDir[4]) begin	//Update if update is pressed, SW[4] = strides of 10
				CursorY <= iCursorDir[3] ? CursorY-10'd10 : 				//SW[3] is Up
							  iCursorDir[2] ? CursorY+10'd10 : CursorY;	//SW[2] is Down
				CursorX <= iCursorDir[1] ? CursorX-10'd10 : 				//SW[1] is Left
							  iCursorDir[0] ? CursorX+10'd10 : CursorX;	//SW[0] is Right
			end
			else if (updateCursor) begin	//Update if update is pressed
				CursorY <= iCursorDir[3] ? CursorY-10'b1 : 				//SW[3] is Up
							  iCursorDir[2] ? CursorY+10'b1 : CursorY;	//SW[2] is Down
				CursorX <= iCursorDir[1] ? CursorX-10'b1 : 				//SW[1] is Left
							  iCursorDir[0] ? CursorX+10'b1 : CursorX;	//SW[0] is Right
			end
			else begin	//Otherwise retain value
				CursorX <= CursorX;
				CursorY <= CursorY;
			end
		end
	end
	
endmodule
