module Wireless (
	iCLK_50,
	iReset,
	iNewFrame,
	iGuitarButs_x,
	iBassButs_x,
	iDrumButs_x,
	oUART_TXD );
	
	input			iCLK_50;
	input			iReset;
	input			iNewFrame;
	input	[7:0]	iGuitarButs_x, iBassButs_x, iDrumButs_x;
	
	output 		oUART_TXD;
	
	parameter baud_rate = 9600 ;
	parameter baud_set = (50000000/(2*baud_rate)) - 1 ; // assumes clock rate of 50e6
	
	parameter 	IDLE=2'd0, SEND=2'd1, WAIT=2'd2;
	
	parameter 	ID_G=4'd0, BUT_G=4'd1,	// Guitar
					ID_B=4'd2, BUT_B=4'd3,	// Bass
					ID_D=4'd4, BUT_D=4'd5;	// Drums
					
	parameter 	DELAY=18'd62500;	// (1/9600)*(9+3 bits)*(50MHz) = 62500 ticks;
	
	reg 				baud_clock;
	reg 	[31:0] 	baud_counter ;
	reg	[3:0]		Status;
	reg	[1:0] 	State;
	reg	[7:0] 	txData;
	reg	[17:0]	txCount;
	reg				startSend, startSend_dly;
	
	wire	newFrame;

	// Determine newFrame w/ 50MHz clock from 27MHz
	newPress np0(iCLK_50, iNewFrame, newFrame);
	
	// Baud clock generator
	always @(posedge iCLK_50) begin
		if (iReset) baud_counter <= 32'd0 ;
		if (baud_counter == baud_set) 
		begin
			baud_counter <= 32'd0 ;
			baud_clock <= ~baud_clock ;
		end
		else baud_counter <= baud_counter + 1 ;
	end
	
	// Serial Transmitter module
	SerialTransmitter uart(
		.CLOCK_50		(iCLK_50), 			// input 50MHz clock
		.UART_TXD		(oUART_TXD), 		// output external serial transmitter line
		.toTransmit		(txData), 			// input Byte to transmit
		.reset			(iReset), 			// input reset signal
		.sendNew			(startSend_dly), 	// input should we send the new data?
		.baud_clock		(baud_clock) 		// input the baud clock
	);
	
	// Selects what data to transmit based on status
	always @(Status) begin
		case (Status)
			ID_G:		txData = 8'hC0;
			ID_B:		txData = 8'h40;
			ID_D:		txData = 8'h80;
			BUT_G:	txData = iGuitarButs_x;
			BUT_B:	txData = iBassButs_x;	
			BUT_D:	txData = iDrumButs_x;
			default:	txData = 8'hFF;
		endcase
	end
	
	always @(posedge iCLK_50) begin
		if (iReset) begin
			State <= IDLE;
			Status <= ID_G;
			txCount <= 18'd0;
			startSend <= 1'b0;
			startSend_dly <= 1'b0;
		end
		else begin
			// Delay startSend 1 cycle to setup the data
			startSend_dly <= startSend;
			
			// State machine to update all devices
			case (State)
				IDLE: begin	// Not transmitting
					//State <= newFrame ? SEND : IDLE;
					//startSend <= newFrame ? 1'b1 : 1'b0;
					State <= SEND;
					startSend <= 1'b1;
				end
				
				SEND: begin	// Initiate a byte transmit
					startSend <= 1'b0;
					State <= WAIT;
					txCount <= 18'd0;
				end
				
				WAIT: begin	// Wait for transmit to finish
					txCount <= txCount + 18'd1;
					if (txCount==DELAY) begin
						Status <= (Status==BUT_D) ? ID_G : (Status + 4'd1);
						//State <= (Status==BUT_D) ? IDLE : SEND;
						//startSend <= (Status==BUT_D) ? 1'b0 : 1'b1;
						State <= SEND;
						startSend <= 1'b1;
					end			
				end
				
				default: begin
					State <= IDLE;
					Status <= ID_G;
				end
			endcase			
		end
	end
	
endmodule
	
	